/**
  Web Client (Ethernet version)
  Name: opta_ethernet_web_client.ino
  Purpose: This sketch connects an Opta device to ip-api.com via Ethernet
  and fetches IP details for the device.

  @author Arduino PRO Content Team
  @version 2.0 15/08/23
*/

// Include the necessary libraries
#include <Ethernet.h>
#include <Arduino_JSON.h>

// Server address for ip-api.com
const char* server = "ip-api.com";

// API endpoint path to get IP details in JSON format
String path = "/json/";

// Static IP configuration for the Opta device
IPAddress ip(10, 130, 22, 84);

// Ethernet client instance for the communication
EthernetClient client;

// JSON variable to store and process the fetched data
JSONVar doc;

// Variable to ensure we fetch data only once
bool dataFetched = false;

void setup() {
  // Begin serial communication at a baud rate of 115200
  Serial.begin(115200);

  // Wait for the serial port to connect,
  // This is necessary for boards that have native USB.
  while (!Serial);

  // Attempt to start Ethernet connection via DHCP,
  // If DHCP failed, print a diagnostic message.
  if (Ethernet.begin() == 0) {
    Serial.println("- Failed to configure Ethernet using DHCP!");

    // Try to configure Ethernet with the predefined static IP address.
    Ethernet.begin(ip);
  }
  delay(2000);
}

void loop() {
  // Ensure we haven't fetched data already
  if (!dataFetched) {
    // Ensure the Ethernet link is active
    if (Ethernet.linkStatus() == LinkON) {
      // Establish a connection to the server
      if (client.connect(server, 80)) {
        // Compose and send the HTTP GET request
        client.print("GET ");
        client.print(path);
        client.println(" HTTP/1.1");
        client.print("Host: ");
        client.println(server);
        client.println("Connection: close");
        client.println();

        // Wait and skip the HTTP headers to get to the JSON data
        char endOfHeaders[] = "\r\n\r\n";
        client.find(endOfHeaders);

        // Read and parse the JSON response
        String payload = client.readString();
        doc = JSON.parse(payload);

        // Check if the parsing was successful
        if (JSON.typeof(doc) == "undefined") {
          Serial.println("- Parsing failed!");
          return;
        }

        // Extract and print the IP details
        Serial.println("*** IP Details:");
        Serial.print("- IP Address: ");
        Serial.println((const char*)doc["query"]);
        Serial.print("- City: ");
        Serial.println((const char*)doc["city"]);
        Serial.print("- Region: ");
        Serial.println((const char*)doc["regionName"]);
        Serial.print("- Country: ");
        Serial.println((const char*)doc["country"]);
        Serial.println("");

        // Mark data as fetched
        dataFetched = true;
      }
      // Close the client connection once done
      client.stop();
    } else {
      Serial.println("- Ethernet link disconnected!");
    }
  }
}
